"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createInlineSpellingDictionary = void 0;
const AutoResolve_js_1 = require("../util/AutoResolve.js");
const util_js_1 = require("../util/util.js");
const createSpellingDictionary_js_1 = require("./createSpellingDictionary.js");
const FlagWordsDictionary_js_1 = require("./FlagWordsDictionary.js");
const IgnoreWordsDictionary_js_1 = require("./IgnoreWordsDictionary.js");
const SpellingDictionaryCollection_js_1 = require("./SpellingDictionaryCollection.js");
const SuggestDictionary_js_1 = require("./SuggestDictionary.js");
const cache = (0, AutoResolve_js_1.createAutoResolveWeakCache)();
function createInlineSpellingDictionary(inlineDict, source) {
    return cache.get(inlineDict, () => {
        const { words, flagWords, ignoreWords, suggestWords, name } = inlineDict;
        const dictSources = [
            words && (0, createSpellingDictionary_js_1.createSpellingDictionary)(words, name + '-words', source, inlineDict),
            flagWords && (0, FlagWordsDictionary_js_1.createFlagWordsDictionary)(flagWords, name + '-flag-words', source),
            ignoreWords && (0, IgnoreWordsDictionary_js_1.createIgnoreWordsDictionary)(ignoreWords, name + '-ignore-words', source),
            suggestWords && (0, SuggestDictionary_js_1.createSuggestDictionary)(suggestWords, name + '-suggest', source),
        ].filter(util_js_1.isDefined);
        return (0, SpellingDictionaryCollection_js_1.createCollection)(dictSources, name, source);
    });
}
exports.createInlineSpellingDictionary = createInlineSpellingDictionary;
//# sourceMappingURL=createInlineSpellingDictionary.js.map